#include "Deghost.h"

int get_position_of_right_most_set_bit_32(unsigned int number)
{
	const char MultiplyDeBruijnBitPosition2[32] = {
		0,  1,  28, 2,  29, 14, 24, 3, 30, 22, 20, 15, 25, 17, 4,  8,
		31, 27, 13, 23, 21, 19, 16, 7, 26, 12, 18, 6,  11, 5,  10, 9
	};
	return MultiplyDeBruijnBitPosition2[(unsigned int)((number & -number) * 0x077CB531U) >> 27];
}

int get_position_of_right_most_set_bit_16(unsigned short number)
{
	static const char MultiplyDeBruijnBitPosition2[16] = {0, 1, 2, 5, 3, 9, 6, 11, 15, 4, 8, 10, 14, 7, 13, 12};
	return MultiplyDeBruijnBitPosition2[(unsigned short)((number & -number) * 0x09AFU) >> 12];
}

int get_position_of_right_most_set_bit_8(unsigned char number)
{
	unsigned int n = number;
	n &= -n;
	n *= n * n;
	// n *= n;
	// n *= 0x34A;
	// n *= 0x17U;
	// n *= 0x53977U;
	n *= 0x53977U << 8;
	n >>= 29;
	return n;
}

int get_position_of_right_most_set_bit_16_f(unsigned char number)
{
	unsigned int n = number;
	double f;
	n &= -n;
	f = n;
	n = *(unsigned int *)&f; // Copy raw value to int
	n >>= 23;
	n -= 127;
	return n;
}

void sprintb(char *string, int number, int bits)
{
	int i;
	for (i = 0; i < bits; i++)
	{
		string[bits - i - 1] = '0' + (number & 1);
		number >>= 1;
	}
	string[bits + 1] = '\0';
}

int getFps()
{
	static int display_fps = 0, fps = 1, time = 0;
	int ticks = RTC_GetTicks();
	if (ticks - time > 128)
	{
		display_fps = fps;
		fps = -1;
		time = ticks;
	}
	fps++;
	return display_fps;
}

double getFps_TMU(unsigned int ticks)
{
	static int fps = 1, ticks_old = 0;
	static double display_fps = 0;
	tmu_t *T = &TMU[TIMER_0];
	// int ticks = T->TCNT;
	unsigned int delta = ticks_old - ticks;
	// static unsigned int delta_old;
	unsigned int OneSecond = 0;
	double test;
	unsigned char string[33];
	static unsigned int Pphi_freq = 0;
	// fps++;

	// if (delta >= OneSecond)
	// {
	if (!Pphi_freq)
		Pphi_freq = Pphi_Freq;
	OneSecond = Pphi_freq / (4 << (T->TCR.TPSC << 1));
	display_fps = (double)OneSecond / (double)delta;
	// 	display_fps = fps;
	// sprintf((char *)string, "%f", display_fps);
	// PrintMini(70, 12, string, 0);
	// 	test = (double)((delta - OneSecond)) / (double)OneSecond;
	// 	display_fps += test;
	// 	fps = 0;
	ticks_old = ticks;
	// sprintf((char *)string, "%u", delta);
	// PrintMini(0, 30, string, 0);
	// sprintf((char *)string, "%u", OneSecond);
	// PrintMini(0, 36, string, 0);
	// 	sprintf((char *)string, "%f", test);
	// 	PrintMini(0, 48, string, 0);
	// 	delta_old = delta;
	// }
	return display_fps;
}

int pow(int number, int power)
{
	int i;
	int output = 1;
	for (i = 0; i < power; i++)
		output *= number;
	return output;
}

int String2Int(char string[])
{
	int i = 0;
	int value = 0;
	while (string[i] != '\0')
		value = value * 10 - '0' + string[i++];
	return value;
}

unsigned int printAndEditFancyNumber(unsigned *number2, int x, int y)
{
	static int key_old = 0;
	char string[11];
	unsigned char character[2] = {0};
	unsigned int number = *number2;
	static int position = 9;
	int key = 0, i;

	position = clamp(position, 0, 9);
	number = clamp(number, 0, 0x7FFFFFFF);
	sprintf(string, "%010u", number);
	for (i = 0; i < 10; i++)
	{
		character[0] = string[i];
		PrintMini(i * 5 + 1 + x, 1 + y, character, position == i ? MINI_REV : MINI_OVER);
		if (position == i)
			ML_vertical_line(i * 5 + x, y, 6 + y, ML_BLACK);
		else
			ML_vertical_line(i * 5 + x, y, 6 + y, ML_WHITE);
	}
	if (Gray.IsSH3)
		GetKey_NB(&key);
	else {
		key = Get_Key();
		key = KeyEncode(key / 10, (key % 10) + 1);
	}
	if (key != key_old)
	{
		switch (key)
		{
		case KEY_CTRL_UP:
			string[position] = rotateUp(string[position], '0', '9');
			number = String2Int(string);
			break;
		case KEY_CTRL_DOWN:
			string[position] = rotateDown(string[position], '0', '9');
			number = String2Int(string);
			break;
		case KEY_CHAR_PLUS:
			number += pow(10, 9 - position);
			break;
		case KEY_CHAR_MINUS:
			number -= pow(10, 9 - position);
			break;
		case KEY_CTRL_LEFT:
		case KEY_CHAR_MULT:
			position--;
			break;
		case KEY_CTRL_RIGHT:
		case KEY_CHAR_DIV:
			position++;
			break;
		case KEY_CTRL_DEL:
			key = KEY_CHAR_0;
		case KEY_CHAR_0:
		case KEY_CHAR_1:
		case KEY_CHAR_2:
		case KEY_CHAR_3:
		case KEY_CHAR_4:
		case KEY_CHAR_5:
		case KEY_CHAR_6:
		case KEY_CHAR_7:
		case KEY_CHAR_8:
		case KEY_CHAR_9:
			string[position] = key;
			number = String2Int(string);
			break;
		case KEY_CTRL_AC:
			number = 300;
			break;
		}
		key_old = key;
		number = clamp(number, 0, 0x7FFFFFFF);
	}
	*number2 = number;
	return key;
}

int AddIn_main(int isAppli, unsigned short OptionNum)
{
	int x = 0, y = 32, i, j = 0, temp = 0;
	int black = 1, white = 0;
	int direction;
	unsigned int key = KEY_PLUS, key2;
	unsigned int ticks = RTC_GetTicks();
	// char DisplayShades[64][128];
	unsigned char string[33];
	int pause = FALSE;
	tmu_t *T;
	unsigned int PFC_Ratio;
	char *vram = GetVRAMAddress();
	int dark = 0x50000, light = 0x500000;
	int amount = 1;
	int test;
	int delta, delta_old;
	int fps;
	double fps_tmu;
	unsigned int number = 64000000, time2 = 0, time3 = 0; //230400;
	register unsigned int time = 0xFFFFFFFF;
	int TMU_freq;
	unsigned int Pphi_freq;
	int array[9] = {0};

	Gray_Init();
	if (!Gray.IsSH3) {
		Gray_Stop_SH4_Timer();
		Gray_Init_SH4_Timer();
		T = &TMU[TIMER_0];
		test = 0xFFFFFFFF;
		// PFC_Ratio = ((CPG.FLLFRQ.BIT.FLF >> CPG.FLLFRQ.BIT.SELXM) * (CPG.FRQCRA.BIT.STC + 1) >> CPG.FRQCRA.BIT.PFC);
		Gray_Start_SH4_Timer();
		// number = Pphi_Freq / 1024;
		// TMU_freq = Pphi_Freq / (4 << (T->TCR.TPSC << 1));
		Pphi_freq = Pphi_Freq;
	}
	while (TRUE)
	{
		while (IsKeyUp(KEY_MENU) && IsKeyUp(KEY_EXIT))
		{

			if (Gray.IsSH3)
			{
				while (RTC_GetTicks() - number < delta)
					Sleep(0);
				delta = RTC_GetTicks();
				fps = getFps();
			}
			else {
				if (T->TCR.TPSC != 5 && T->TCR.TPSC != 7)
				{
					vram = GetVRAMAddress();
					for (y = 0; y < GRAY_SCREEN_HEIGHT; y++)
					{
						for (i = 0; i < 16; i++)
						{
							if (i > 8)
								*vram++ = (x == abs(white)) ^ (white < 0) ? 0x00 : 0xFF;
								// *vram++ = (white < 0) ^ (x == 0) ? 0xFF : 0x00;
							else
								*vram++;
						}
					}
					while ((delta = T->TCNT) > time)
						i++;
					while (T->TCNT == time);
					ML_display_vram();
					fps_tmu = getFps_TMU(delta);
					test = (unsigned int)(((double)(Pphi_freq / (4 << (T->TCR.TPSC << 1))) * 1000000.0) / (double)number);
					time = delta - test;
				}
			}

			if (IsKeyDown(KEY_EXP) && !Gray.IsSH3)
			{
				if (key != KEY_EXP)
				{
					Gray_Pause_SH4_Timer();
					T->TCR.TPSC += 1;
					if (T->TCR.TPSC > 0x3)
						T->TCR.TPSC = 0;
					Gray_Start_SH4_Timer();
					sprintf((char *)string, "Pphi %d ", T->TCR.TPSC);
					PrintMini(0, 40, string, 0);
					key = KEY_EXP;
				}
			}
			else if (IsKeyDown(KEY_TAN)) {
				if (key != KEY_TAN)
				{
				white++;
				key = KEY_TAN;
				}
			}
			else if (IsKeyDown(KEY_STORE))
			{
				if (key != KEY_STORE)
				{
					white--;
					key = KEY_STORE;
				}
			}
			else
				key = 0;

			x++;
			if (x > abs(white))
				x = 0;

			if (Gray.IsSH3)
			{
				sprintf((char *)string, "%X", RTC_GetTicks());
				PrintMini(0, 6, string, 0);
				sprintf((char *)string, "%d   ", fps);
				PrintMini(0, 0, string, 0);
				sprintf((char *)string, "%X       ", delta - delta_old);
				PrintMini(0, 12, string, 0);
			}
			else
			{
				sprintf((char *)string, "FPS:%f   ", fps_tmu);
				PrintMini(0, 0, string, 0);
				sprintf((char *)string, "%u       ", delta_old - delta);
				PrintMini(0, 6, string, 0);
				sprintf((char *)string, "%u       ", i);
				PrintMini(0, 12, string, 0);
				sprintf((char *)string, "%d       ", white);
				PrintMini(0, 18, string, 0);
				// sprintf((char *)string, "%u       ", Pphi_freq);
				// PrintMini(0, 18, string, 0);
				// sprintf((char *)string, "%X", T->TCNT);
				// PrintMini(0, 0, string, 0);
				// sprintf((char *)string, "%u       ", Pphi_freq);
				// PrintMini(0, 12, string, 0);
				// sprintf((char *)string, "%u       ", test);
				// PrintMini(0, 18, string, 0);
				// sprintf((char *)string, "%u       ", delta - time);
				// PrintMini(0, 24, string, 0);
				// sprintf((char *)string, "%u   ", Get_Key());
				// PrintMini(110, 58, string, 0);
			}
			// sprintf((char *)string, "%f", (double)(Pphi_Freq / (4 << (T->TCR.TPSC << 1))));
			// PrintMini(0, 30, string, 0);
			// sprintf((char *)string, "%X       ", Pphi_Freq);
			// PrintMini(0, 12, string, 0);
			// sprintf((char *)string, "%u       ", Pphi_Freq / (delta_old - delta));
			// PrintMini(0, 18, string, 0);


			// if (IsKeyDown(KEY_RPAR))
			// 	black = 1;
			// if (IsKeyDown(KEY_LPAR))
			// 	black = 0;
			// if (black)
			printAndEditFancyNumber(&number, 0, 57);
			// else
			// printAndEditFancyNumber(&time3, 0, 50);

			// vram = GetVRAMAddress();
			// for (y = 0; y < GRAY_SCREEN_HEIGHT; y++)
			// {
			// 	for (i = 0; i < 16; i++)
			// 	{
			// 		if (i > 8)
			// 			*vram++ = 0xFF;
			// 		else
			// 			*vram++;
			// 	}
			// }

			// if (!Gray.IsSH3)
			// {
			// 	if (T->TCR.TPSC != 5 && T->TCR.TPSC != 7)
			// 	{
			// 		time2 = delta - (time3 / (4 << (T->TCR.TPSC << 1)));

			// 		while (T->TCNT >= time2);
			// 		ML_display_vram();
			// 	}
			// }
			// else
			// ML_display_vram();

			delta_old = delta;

			if (IsKeyDown(KEY_EXE))
			{
				while (IsKeyDown(KEY_EXE));
				do {
					vram = GetVRAMAddress();
					for (y = 0; y < GRAY_SCREEN_HEIGHT; y++)
						for (i = 0; i < 16; i++)
							*vram++ = ((i < 8 ? array[9 - i] : !array[i - 8])) ? 0x00 : 0xFF;
					while ((delta = T->TCNT) > time);
					while (T->TCNT == time);
					ML_display_vram();
					test = (unsigned int)(((double)(Pphi_freq / (4 << (T->TCR.TPSC << 1))) * 1000000.0) / (double)number);
					time = delta - test;
					delta_old = delta;
					for (i = 0; i < 9; i++)
					{
						if (array[i] > (i >> 1))
							array[i] = 0;
						else
							array[i]++;
					}
				} while (IsKeyUp(KEY_EXE));
				while (IsKeyDown(KEY_EXE));
			}
			
		}
		while (IsKeyDown(KEY_MENU) || IsKeyDown(KEY_EXIT));
		Keyboard_ClrBuffer();
		Gray_Pause_SH4_Timer();
		GetKey(&key);
		Gray_Start_SH4_Timer();
	}
}

#pragma section _BR_Size
unsigned long BR_Size;
#pragma section
#pragma section _TOP
int InitializeSystem(int isAppli, unsigned short OptionNum)
{
	return INIT_ADDIN_APPLICATION(isAppli, OptionNum);
}
#pragma section