#ifndef _GRAYH
#define _GRAYH

#include <fxlib.h>
#include <timer.h>
#include "Syscall.h"

// #define Gray_vram_adress (*(Gray_sc_cpv_)Gray_SC0135)
// typedef char* (*Gray_sc_cpv_)(void);
// const unsigned int Gray_SC0135[] = { 0xD201D002, 0x422B0009, 0x80010070, 0x0135 };
#define Gray_VRAM_Address() (char *)VRam_Base()

#define GRAY_SCREEN_WIDTH 128
#define GRAY_SCREEN_HEIGHT 64
typedef enum Boolean { GRAY_FALSE, GRAY_TRUE } Gray_Bool;
typedef enum Colour { GRAY_TRANSPARENT = -1, GRAY_WHITE, GRAY_BLACK, GRAY_XOR, GRAY_CHECKER, GRAY_LIGHT, GRAY_DARK, GRAY_INVERTED, GRAY_CHECKER_GRAY } Gray_Colour;

struct GrayScale {
	Gray_Bool Initialised;
	int IsSH3;
	int IsGraph35eII;
	unsigned int Time;

	struct DisplayDriverBuffers {
		char* Black;
		char* Dark;
		char* Light;
	} DD;

	struct VRAMBuffers {
		char* Black;
		char* Dark;
		char* Light;
	} VRAM;

// } Gray = {GRAY_FALSE};
};

extern struct GrayScale Gray;

// https://gitea.planet-casio.com/Lephenixnoir/gint/src/branch/master/include/gint/mpu/tmu.h

/* tmu_t - a single timer from a standard timer unit */
typedef volatile struct
{
	unsigned int TCOR;			/* Constant register */
	unsigned int TCNT;			/* Counter register, counts down */

	struct TimerControlRegister
	{
		// unsigned short word;
		unsigned short		:7;
		unsigned short UNF	:1;	/* Underflow flag */
		unsigned short		:2;
		unsigned short UNIE	:1;	/* Underflow interrupt enable */
		unsigned short CKEG	:2;	/* Input clock edge */
		unsigned short TPSC	:3;	/* Timer prescaler (input clock) */
	} TCR;

} tmu_t;

typedef volatile struct
{
	char TSTR;			/* Timer Start Register */
	char pad8;
	short pad16;

	tmu_t TMU[1];

} sh7305_tmu_t;

#define SH7305_TMU (*((sh7305_tmu_t *)0xA4490004))

/* Standard timer */
static tmu_t *TMU = SH7305_TMU.TMU;
/* TSTR register for standard timers */
static volatile char *TSTR = &SH7305_TMU.TSTR;

struct st_cpg
{
	union								// 0xA41500000
	{									// struct FRQCRA   similar to SH7724
		unsigned int LONG;				// long Word Access
		struct							// Bit  Access
		{
			unsigned int KICK	: 1;
			unsigned int		: 1;
			unsigned int STC	: 6;
			unsigned int IFC	: 4;
			unsigned int		: 4;
			unsigned int SFC	: 4;
			unsigned int BFC	: 4;
			unsigned int		: 4;
			unsigned int PFC	: 4;
		} BIT;
	} FRQCRA;

	unsigned int FRQCRB;			// 0xA41500004
	unsigned int FCLKACR;			// 0xA41500008
	unsigned int FCLKBCR;			// 0xA4150000C
	unsigned int unknownA41500010;
	unsigned int unknownA41500014;
	unsigned int IRDACLKCR;			// 0xA41500018
	unsigned int unknownA4150001C;
	unsigned int unknownA41500020;
	union							// 0xA41500024
	{								// struct FRQCRA   similar to SH7724
		unsigned int LONG;			// long Word Access
		struct						// Bit  Access
		{
			unsigned int		: 17;
			unsigned int PLLE	: 1;	// PLL Enable
			unsigned int		: 1;
			unsigned int FLLE	: 1;	// FLL Enable
			unsigned int		: 10;
			unsigned int CKOFF	: 1;	// CKO Output Stop
			unsigned int		: 1;
		} BIT;
	} PLLCR;
	unsigned int unknownA41500028;
	unsigned int unknownA4150002C;
	unsigned int unknownA41500030;
	unsigned int unknownA41500034;
	unsigned int unknownA41500038;
	unsigned int SPUCLKCR;			// 0xA4150003C
	unsigned int unknownA41500040;
	unsigned int unknownA41500044;
	unsigned int VCLKCR;			// 0xA41500048
	unsigned int unknownA4150004C;

	union
	{						// struct FLLFRQ   similar to SH7724
		unsigned int LONG;	// long Word Access
		struct
		{								// Bit  Access
			unsigned int		: 16;	//
			unsigned int SELXM	: 2;	// SELXM
			unsigned int		: 3;	//
			unsigned int FLF	: 11;	// FLF
		} BIT;
	} FLLFRQ;

	unsigned int unknownA41500054;
	unsigned int unknownA41500058;
	unsigned int unknownA4150005C;
	unsigned int LSTATS;
};

#define CPG (*(volatile struct st_cpg *)0xA4150000)					// FRQCRA    Address


/* Clock input */
enum TimerPrescaler
{
	TIMER_Pphi_4,		// 0x384000		3686400
	TIMER_Pphi_16,		// 0x E1000		921600
	TIMER_Pphi_64,		// 0x 38400		230400
	TIMER_Pphi_256,		// 0x  E100		57600
						// 0x  3840		14400
	TIMER_TCLK = 5,		// 0x     0		0
						// 0x  8000		32768
};

/* Clock edge */
enum ClockEdge
{
	TIMER_COUNT_RISING_EDGE,
	TIMER_COUNT_FALLING_EDGE,
	TIMER_COUNT_DUAL_EDGE
};

/* Clock channel */
enum TMUChannel
{
	TIMER_0,
	TIMER_1,
	TIMER_2
};

#define Pphi_Freq ((32768 * (CPG.PLLCR.BIT.FLLE ? CPG.FLLFRQ.BIT.FLF >> CPG.FLLFRQ.BIT.SELXM : 1) * (CPG.PLLCR.BIT.PLLE ? CPG.FRQCRA.BIT.STC + 1 : 1)) >> (CPG.FRQCRA.BIT.PFC + 1))

#define SH7705_EXTRA_TMU_COUNT *(volatile unsigned int *)0xA44C0038 //  ETMU: Used by SetTimer() and keyboard repeats. 32768Hz 819 down counter. SH3
#define P7305_EXTRA_TMU0_COUNT *(volatile unsigned int *)0xA44D0038 // ETMU0: Used by SetTimer() and keyboard repeats. 32768Hz 819 down counter. SH4
#define P7305_EXTRA_TMU5_COUNT *(volatile unsigned int *)0xA44D00D8 // ETMU5: 32768Hz down counter. SH4


void Gray_Init(void);	//Initialises the buffers
int Gray_Start(void);	//Start the gray engine
int Gray_Stop(void);	//Pause the gray engine
int Gray_Toggle(void);	//Toggle the gray engine
// void Gray_Init_SH4_Timer(void);
void Gray_Start_SH4_Timer(void);
void Gray_Pause_SH4_Timer(void);
void Gray_Stop_SH4_Timer(void);
void Gray_PowerOffCalc(Gray_Bool logo);								//Turns off calculator
void Gray_Copy_Buffer(char fromBuffer[1024], char toBuffer[1024]);	//Copy one buffer to another
void Gray_Swap_Buffers(void);	//Swap two buffers around
void Gray_Display_GrayScale(void);		//Display the DisplayDriver buffers to the display
void Gray_Delay_ASM(int instructions);		//Create delay by looping. Instructions * 4
void Gray_Display_Buffer_DD(char buffer[1024]);
// void Gray_Display_Buffer_DD(char buffer[1024], int row, int LCD_reg_sel_1, int LCD_data_reg, register int LCD_reg_sel_2);
// void Gray_Display_Buffer_DD(int buffer[1024], int row, int LCD_reg_sel_1, int LCD_data_reg, register int LCD_reg_sel_2);
// void Gray_Display_DD_ASM(char* buffer, int OS);		//Display a buffer to the screen
// void Gray_Pixel(int x, int y, Gray_Colour colour);							//Draw a gray pixel
// void Gray_Vertical_Line(int x, int y1, int y2, Gray_Colour colour);			//Draw a gray vertical line
// void Gray_Horizontal_Line(int y, int x1, int x2, Gray_Colour colour);		//Draw a gray horizontal line
// void Gray_Rectangle(int x1, int y1, int x2, int y2, Gray_Colour colour);	//Draw a gray Rectangle
// void Gray_BMP_Load(unsigned char* bmp, int x, int y, int width, int height, int fullWidthInBytes);
// void Gray_Mono_VRAM_Function_To_Gray(void (*function)(void));

#endif