#include "Gray.h"

struct GrayScale Gray = {GRAY_FALSE, GRAY_TRUE, GRAY_FALSE};

void Gray_Init(void)
{
	char OS[10];
	// static char buffer_DD_Black[1024]	= { 0 };
	// static char buffer_DD_Dark[1024]	= { 0 };
	// static char buffer_DD_Light[1024]	= { 0 };
	// static char buffer_VRAM_Black[1024]	= { 0 };
	// static char buffer_VRAM_Dark[1024]	= { 0 };
	// static char buffer_VRAM_Light[1024]	= { 0 };
	// Gray.DD.Dark	= buffer_DD_Dark;
	// Gray.DD.Light	= buffer_DD_Light;
	// Gray.VRAM.Dark	= buffer_VRAM_Dark;
	// Gray.VRAM.Light	= buffer_VRAM_Light;
	// Gray.DD.Black	= malloc(1024);
	// Gray.DD.Dark	= malloc(1024);
	// Gray.DD.Light	= malloc(1024);
	// Gray.VRAM.Black	= malloc(1024);
	// Gray.VRAM.Dark	= malloc(1024);
	// Gray.VRAM.Light	= malloc(1024);
	System_GetOSVersion(OS);
	Gray.IsSH3			= OS[1] == '1';
	Gray.IsGraph35eII	= OS[1] == '3';
	Gray.Initialised	= GRAY_TRUE;
	if (!Gray.IsSH3)
		Gray.Initialised = !Gray_Init_SH4_Timer();
	// SetQuitHandler((void *)Gray_Stop);
}

// -1 = Error : 1 = Started
int Gray_Start(void) {
	if (Gray.Initialised != GRAY_TRUE)
		Gray_Init();
	DisableGetkeyToMainFunctionReturn();
	// if(!Gray.IsSH3)
	// 	Gray_Start_SH4_Timer();
	return SetTimer(ID_USER_TIMER5, 25, Gray_Display_GrayScale) == -1 ? -1 : 1;
}

//-2 = Error : 0 = Stopped
int Gray_Stop(void)
{
	EnableGetkeyToMainFunctionReturn();
	if (!Gray.IsSH3)
		Gray_Stop_SH4_Timer();
	return KillTimer(ID_USER_TIMER5) == -2 ? -2 : 0;
}

//-1 = Error : 0 = Stopped : 1 = Started
int Gray_Toggle(void) {
	if (Gray_Stop() == 0)
		return 0;
	else
		return Gray_Start();
}

int Gray_Init_SH4_Timer(void)
{
		/* Refuse to setup timers that are already in use */
		tmu_t *T = &TMU[TIMER_0];
		if (T->TCR.UNIE || *TSTR & 1 << TIMER_0)
		{
			// Gray.Initialised = GRAY_FALSE;
			return 1;
		}

		ML_pixel(40, 40, 1);

		/* Configure the counter, clear interrupt flag*/
		T->TCOR = 0xFFFFFFFF;
		T->TCNT = 0xFFFFFFFF;
		// T->TCR.TPSC = TIMER_TCLK;
		T->TCR.TPSC = TIMER_Pphi_16;
		do T->TCR.UNF = GRAY_FALSE;
		while(T->TCR.UNF);

		/* Disable interrupt and count on rising edge (SH7705) */
		T->TCR.UNIE = GRAY_FALSE;
		T->TCR.CKEG = TIMER_COUNT_DUAL_EDGE;
		return 0;
}

void Gray_Start_SH4_Timer(void)
{
	*TSTR |= 1 << TIMER_0;
}

void Gray_Pause_SH4_Timer(void)
{
	*TSTR &= -2 << TIMER_0;
}

void Gray_Stop_SH4_Timer(void)
{
	tmu_t *T = &TMU[TIMER_0];
	Gray_Pause_SH4_Timer();
	// do
	// 	TMU[TIMER_0].TCR.word = 0;
	// while (TMU[TIMER_0].TCR.word);
	T->TCR.UNIE = GRAY_FALSE;
	T->TCOR = 0xFFFFFFFF;
	T->TCNT = 0xFFFFFFFF;
}

void Gray_Setup_Light_Buffer(void)
{
	int i;
	for (i = 0; i < 1024; i++)
		Gray.VRAM.Light[i] |= Gray.VRAM.Dark[i];
}

void Gray_Setup_Black_Buffer(void)
{
	int i;
	for (i = 0; i < 1024; i++)
		Gray.VRAM.Black[i] = Gray.VRAM.Dark[i] & Gray.VRAM.Light[i];
}

void Gray_Swap_Buffers(void) {
	// char* tempBufferBlack = Gray.DD.Black;
	char *tempBufferBlack = Gray.DD.Black;
	char* tempBufferDark = Gray.DD.Dark;
	char* tempBufferLight = Gray.DD.Light;
	char byte;
	int i;

	Gray_Setup_Black_Buffer();
	Gray_Setup_Light_Buffer();

	// for (i = 0; i < 1024; i += 4)
	// {
	// 	byte = Gray.VRAM.Black[i + 0];
	// 	Gray.VRAM.Black[i + 0] = Gray.VRAM.Black[i + 3];
	// 	Gray.VRAM.Black[i + 3] = byte;
	// 	byte = Gray.VRAM.Black[i + 1];
	// 	Gray.VRAM.Black[i + 1] = Gray.VRAM.Black[i + 2];
	// 	Gray.VRAM.Black[i + 2] = byte;

	// 	byte = Gray.VRAM.Dark[i + 0];
	// 	Gray.VRAM.Dark[i + 0] = Gray.VRAM.Dark[i + 3];
	// 	Gray.VRAM.Dark[i + 3] = byte;
	// 	byte = Gray.VRAM.Dark[i + 1];
	// 	Gray.VRAM.Dark[i + 1] = Gray.VRAM.Dark[i + 2];
	// 	Gray.VRAM.Dark[i + 2] = byte;

	// 	byte = Gray.VRAM.Light[i + 0];
	// 	Gray.VRAM.Light[i + 0] = Gray.VRAM.Light[i + 3];
	// 	Gray.VRAM.Light[i + 3] = byte;
	// 	byte = Gray.VRAM.Light[i + 1];
	// 	Gray.VRAM.Light[i + 1] = Gray.VRAM.Light[i + 2];
	// 	Gray.VRAM.Light[i + 2] = byte;
	// }
	Gray.DD.Black = Gray.VRAM.Black;
	Gray.DD.Dark = Gray.VRAM.Dark;
	Gray.DD.Light = Gray.VRAM.Light;
	Gray.VRAM.Black = tempBufferBlack;
	Gray.VRAM.Dark = tempBufferDark;
	Gray.VRAM.Light = tempBufferLight;
}

void Gray_Copy_Buffer(char fromBuffer[1024], char toBuffer[1024]) {
	int i;
	for (i = 0; i < 1024; i++)
		toBuffer[i] = fromBuffer[i];
}

void Gray_Display_Buffer_DD(char buffer[1024])
{
	register volatile char *LCD_register_selector = (char *)0xB4000000;
	register volatile char *LCD_data_register = (char *)0xB4010000;
	register char XY_Register = 4, Data_Buffer = 7;
	register char col = 0, row = 0xC0;
	register int i;

	if (Gray.IsGraph35eII){
		Data_Buffer = 10;
		XY_Register = 8;
		row = 0x80;
		col = 4;
	}

	for (i = 0; i < GRAY_SCREEN_HEIGHT; i++) {
		*LCD_register_selector = XY_Register;	// 4	:	8
		*LCD_data_register = row++;				// 0xC0 :	0x80
		*LCD_register_selector = XY_Register;	// 4	:	8
		*LCD_data_register = col;				// 0	:	4
		*LCD_register_selector = Data_Buffer;	// 7	:	10
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
		*LCD_data_register = *buffer++;
	}
}

void Gray_Display_GrayScale(void) {
	unsigned int TimeDark;
	unsigned int TimeLight;
	unsigned int TimeTMU;
	unsigned int TimeETMU;
	unsigned int TimeASM = Gray.Time;
	unsigned int PFC_Ratio;
	tmu_t *T;
	// if (!Gray.IsSH3)
	// 	TimeETMU = P7305_EXTRA_TMU5_COUNT - Gray.Time;
	if (!Gray.IsSH3)
	{
		T = &TMU[TIMER_0];
		T->TCNT = 0xFFFFFFFF;
		Gray_Start_SH4_Timer();
		PFC_Ratio = ((CPG.FLLFRQ.BIT.FLF >> CPG.FLLFRQ.BIT.SELXM) * (CPG.FRQCRA.BIT.STC + 1) >> CPG.FRQCRA.BIT.PFC);

		// TimeTMU = T->TCNT - (Gray.Time * PFC_Ratio / 1800);
		// TimeTMU = 0xFFFFFFFF - (Gray.Time * PFC_Ratio / 1800);
		TimeDark = 0xFFFFFFFF - ((Gray.Time / 100000) * PFC_Ratio / 180);
		TimeLight = 0xFFFFFFFF - ((Gray.Time % 100000) * PFC_Ratio / 180);
		if (TimeLight < TimeDark)
			TimeDark = TimeLight;
	}

	Gray_Display_Buffer_DD(Gray.DD.Light);
	// if (Gray.IsGraph35eII)
	// 	Gray_Display_Buffer_DD(Gray.DD.Light, 0x80, 8, 4, 10);
	// else
	// 	Gray_Display_Buffer_DD(Gray.DD.Light, 0xC0, 4, 0, 7);
	//Gray_Display_DD_ASM(Gray.DD.Light, 0);

	// TimeRTC = Gray.Time / 1000;
	if (Gray.IsSH3)
	{
		while (SH7705_EXTRA_TMU_COUNT > Gray.Time);
		// Delay_ASM(TimeASM);
	}
	else
	{
		// if(Gray.IsGraph35eII) {
		// while (T->TCNT > TimeTMU);
		while (T->TCNT > TimeLight);
		Gray_Display_Buffer_DD(Gray.DD.Dark);
		while (T->TCNT > TimeDark);
		Gray_Pause_SH4_Timer();
		// }
		// else
		// {
		// 	while (T->TCNT > TimeTMU);
		// 	Gray_Pause_SH4_Timer();
		// }
		// while (P7305_EXTRA_TMU0_COUNT > Gray.Time);
		// while (P7305_EXTRA_TMU5_COUNT > TimeETMU);
		// Delay_ASM(TimeASM);

		// TimeRTC = *(volatile unsigned int *)(0xA44D00D8) - Gray.Time / 1000;
		// TimeRTC = 0xFFFFFFFF - Gray.Time;
		// CPG.FRQCRA.BIT.PFC;
		// while (T->TCNT > TimeTMU || *(volatile unsigned int *)0xA44D00D8 > TimeETMU);
	}
	// while (*(volatile unsigned int *)0xA44D0038 > TimeRTC);

	// if (Gray.IsGraph35eII)
	// 	Gray_Display_Buffer_DD(Gray.DD.Dark, 0x80, 8, 4, 10);
	// // Gray_Display_Buffer_DD((int*)Gray.DD.Dark, 0x80, 8, 4, 10);
	// else
	// 	//Gray_Display_DD_ASM(Gray.DD.Dark, 0);
	// 	Gray_Display_Buffer_DD(Gray.DD.Dark, 0xC0, 4, 0, 7);
	// 	// Gray_Display_Buffer_DD((int*)Gray.DD.Dark, 0xC0, 4, 0, 7);

	Gray_Display_Buffer_DD(Gray.DD.Black);

	// Delay_ASM(TimeDark * 1000);
	// if (Gray.OS[1] == '1')
	// 	while (*(volatile unsigned int *)0xA44C0038 > TimeDark);
	// else
	// 	while (*(volatile unsigned int *)0xA44D0038 > TimeDark);

	// if (Gray.OS[1] == '3')
	// 	Gray_Display_Buffer_DD((int*)Gray.DD.Black, 0x80, 8, 4, 10);
	// else
	// 	Gray_Display_Buffer_DD((int*)Gray.DD.Black, 0xC0, 4, 0, 7);

	// 278
	// 302
}

void Gray_Pixel(int x, int y, Gray_Colour colour) {
	int pos;
	if (x & ~127 || y & ~63)
		return;
	pos = y + (x >> 3);
	switch (colour) {
		case GRAY_BLACK:
			Gray.VRAM.Dark[pos] |= GRAY_SCREEN_WIDTH >> (x & 7);
			Gray.VRAM.Light[pos] |= GRAY_SCREEN_WIDTH >> (x & 7);
			break;
		case GRAY_WHITE:
			Gray.VRAM.Dark[pos] &= ~(GRAY_SCREEN_WIDTH >> (x & 7));
			Gray.VRAM.Light[pos] &= ~(GRAY_SCREEN_WIDTH >> (x & 7));
			break;
		case GRAY_XOR:
			Gray.VRAM.Dark[pos] ^= GRAY_SCREEN_WIDTH >> (x & 7);
			Gray.VRAM.Light[pos] |= Gray.VRAM.Dark[pos] & (GRAY_SCREEN_WIDTH >> (x & 7));
			Gray.VRAM.Light[pos] &= Gray.VRAM.Dark[pos] | ~(GRAY_SCREEN_WIDTH >> (x & 7));
			break;
		case GRAY_CHECKER:
			if (y & 1 ^ x & 1) {
				Gray.VRAM.Dark[pos] &= ~(GRAY_SCREEN_WIDTH >> (x & 7));
				Gray.VRAM.Light[pos] &= ~(GRAY_SCREEN_WIDTH >> (x & 7));
			} else {
				Gray.VRAM.Dark[pos] |= GRAY_SCREEN_WIDTH >> (x & 7);
				Gray.VRAM.Light[pos] |= GRAY_SCREEN_WIDTH >> (x & 7);
			}
			break;
	}
}
