#include "MonochromeLib.h"

void ML_clear_vram()
{
	int i, end, *pointer_long, vram;
	char *pointer_byte;
	vram = (int)ML_vram_address();
	end = 4 - vram & 3;
	pointer_byte = (char *)vram;
	for (i = 0; i < end; i++)
		pointer_byte[i] = 0;
	pointer_long = (int *)(vram + end);
	for (i = 0; i < 255; i++)
		pointer_long[i] = 0;
	pointer_byte += 1020 + end;
	end = vram & 3;
	for (i = 0; i < end; i++)
		pointer_byte[i] = 0;
}

void ML_display_vram()
{
	char *LCD_register_selector = (char *)0xB4000000;
	char *LCD_data_register = (char *)0xB4010000;
	char *vram = ML_vram_address();
	char OS[10];
	int i, j;

	System_GetOSVersion(OS);

	if (OS[1] == '3')
	{
		for (i = 0; i < 64; i++)
		{
			*LCD_register_selector = 8;
			*LCD_data_register = i | 128;
			*LCD_register_selector = 8;
			*LCD_data_register = 4;
			*LCD_register_selector = 10;
			for (j = 0; j < 16; j++)
				*LCD_data_register = *vram++;
		}
	}
	else
	{
		for (i = 0; i < 64; i++)
		{
			*LCD_register_selector = 4;
			*LCD_data_register = i | 192;
			*LCD_register_selector = 4;
			*LCD_data_register = 0;
			*LCD_register_selector = 7;
			for (j = 0; j < 16; j++)
				*LCD_data_register = *vram++;
		}
	}
}

void ML_pixel(int x, int y, ML_Colour colour)
{
	char *vram = ML_vram_address();
	if (x & ~127 || y & ~63)
		return;
	switch (colour)
	{
	case ML_BLACK:
		vram[(y << 4) + (x >> 3)] |= 128 >> (x & 7);
		break;
	case ML_WHITE:
		vram[(y << 4) + (x >> 3)] &= ~(128 >> (x & 7));
		break;
	}
}

int ML_pixel_get(int x, int y)
{
	char *vram = ML_vram_address();
	if (x & ~127 || y & ~63)
		return 0;
	return (vram[(y << 4) + (x >> 3)] >> (7 - (x & 7))) & 1;
}

void ML_vertical_line(int x, int y1, int y2, ML_Colour colour)
{
	int i, j;
	char checker, byte;
	char *vram = ML_vram_address();
	if (x & ~127 || (y1 < 0 && y2 < 0) || (y1 > 63 && y2 > 63))
		return;
	if (y1 > y2)
	{
		int tmp = y1;
		y1 = y2;
		y2 = tmp;
	}
	if (y1 < 0)
		y1 = 0;
	if (y2 > 63)
		y2 = 63;

	i = (y1 << 4) + (x >> 3);
	j = (y2 << 4) + (x >> 3);
	switch (colour)
	{
	case ML_BLACK:
		byte = 128 >> (x & 7);
		for (; i <= j; i += 16)
			vram[i] |= byte;
		break;
	case ML_WHITE:
		byte = ~(128 >> (x & 7));
		for (; i <= j; i += 16)
			vram[i] &= byte;
		break;
	}
}

void ML_vertical_line_wrap(int x, int y1, int y2, ML_Colour colour)
{
	int i, j;
	char checker, byte;
	char *vram = ML_vram_address();

	i = (y1 << 4) + ((x >> 3) & 0xF);
	j = (y2 << 4) + ((x >> 3) & 0xF);
	switch (colour)
	{
	case ML_BLACK:
		byte = 128 >> (x & 7);
		for (; i <= j; i += 16)
			vram[i & 0x3FF] |= byte;
		break;
	case ML_WHITE:
		byte = ~(128 >> (x & 7));
		for (; i <= j; i += 16)
			vram[i] &= byte;
		break;
	}
}

void ML_horizontal_line(int y, int x1, int x2, ML_Colour colour)
{
	int i;
	char checker;
	char *vram = ML_vram_address();
	if (y & ~63 || (x1 < 0 && x2 < 0) || (x1 > 127 && x2 > 127))
		return;
	if (x1 > x2)
	{
		i = x1;
		x1 = x2;
		x2 = i;
	}
	if (x1 < 0)
		x1 = 0;
	if (x2 > 127)
		x2 = 127;
	switch (colour)
	{
	case ML_BLACK:
		if (x1 >> 3 != x2 >> 3)
		{
			vram[(y << 4) + (x1 >> 3)] |= 255 >> (x1 & 7);
			vram[(y << 4) + (x2 >> 3)] |= 255 << 7 - (x2 & 7);
			for (i = (x1 >> 3) + 1; i<x2>> 3; i++)
				vram[(y << 4) + i] = 255;
		}
		else
			vram[(y << 4) + (x1 >> 3)] |= (255 >> (x1 % 8 + 7 - x2 % 8)) << (7 - (x2 & 7));
		break;
	case ML_WHITE:
		if (x1 >> 3 != x2 >> 3)
		{
			vram[(y << 4) + (x1 >> 3)] &= 255 << 8 - (x1 & 7);
			vram[(y << 4) + (x2 >> 3)] &= 255 >> 1 + (x2 & 7);
			for (i = (x1 >> 3) + 1; i<x2>> 3; i++)
				vram[(y << 4) + i] = 0;
		}
		else
			vram[(y << 4) + (x1 >> 3)] &= (255 << 8 - (x1 & 7)) | (255 >> 1 + (x2 & 7));
		break;
	}
}

void ML_horizontal_line_wrap(int y, int x1, int x2, ML_Colour colour)
{
	int i;
	char checker;
	char *vram = ML_vram_address();
	switch (colour)
	{
	case ML_BLACK:
		if (x1 >> 3 != x2 >> 3)
		{
			vram[((y << 4) & 0x3FF) + ((x1 >> 3) & 0xF)] |= 255 >> (x1 & 7);
			vram[((y << 4) & 0x3FF) + ((x2 >> 3) & 0xF)] |= 255 << 7 - (x2 & 7);
			for (i = (x1 >> 3) + 1; i < x2 >> 3; i++)
				vram[((y << 4) & 0x3FF) + (i & 0xF)] = 255;
		}
		else
			vram[(y << 4) + (x1 >> 3)] |= (255 >> (x1 % 8 + 7 - x2 % 8)) << (7 - (x2 & 7));
		break;
	case ML_WHITE:
		if (x1 >> 3 != x2 >> 3)
		{
			vram[((y << 4) & 0x3FF) + ((x1 >> 3) & 0xF)] &= 255 << 8 - (x1 & 7);
			vram[((y << 4) & 0x3FF) + ((x2 >> 3) & 0xF)] &= 255 >> 1 + (x2 & 7);
			for (i = (x1 >> 3) + 1; i<x2>> 3; i++)
				vram[((y << 4) & 0x3FF) + (i & 0xF)] = 0;
		}
		else
			vram[(y << 4) + (x1 >> 3)] &= (255 << 8 - (x1 & 7)) | (255 >> 1 + (x2 & 7));
		break;
	}
}

void ML_rectangle(int x1, int y1, int x2, int y2, int border_width, ML_Colour colour)
{
	int i;

	if (x1 > x2)
	{
		i = x1;
		x1 = x2;
		x2 = i;
	}

	if (y1 > y2)
	{
		i = y1;
		y1 = y2;
		y2 = i;
	}

	for (i = 0; i < border_width; i++)
	{
		//ML_horizontal_line(y1 + i, x1 + i, x2 - i, colour);
		//ML_horizontal_line(y2 - i, x1 + i, x2 - i, colour);
		ML_horizontal_line(y1 + i, x1, x2, colour);
		ML_horizontal_line(y2 - i, x1, x2, colour);
		ML_vertical_line(x1 + i, y1 + i + 1, y2 - i - 1, colour);
		ML_vertical_line(x2 - i, y1 + i + 1, y2 - i - 1, colour);
	}
}

void ML_rectangle_wrap(int x1, int y1, int x2, int y2, ML_Colour colour)
{
	int i;

	if (y1 > y2)
	{
		i = y1;
		y1 = y2;
		y2 = i;
	}

	ML_horizontal_line_wrap(y1, x1, x2, colour);
	ML_horizontal_line_wrap(y2, x1, x2, colour);
	ML_vertical_line_wrap(x1, y1 + 1, y2 - 1, colour);
	ML_vertical_line_wrap(x2, y1 + 1, y2 - 1, colour);
}

void ML_bmp_16_or_cl(const unsigned short *bmp, int x, int y)
{
	unsigned long line;
	char i, shift, begin = 0, end = 16, bool1 = 1, bool2 = 1, bool3 = 1, *screen, *p = (char *)&line + 1;
	if (!bmp || x < -15 || x > 127 || y < -15 || y > 63)
		return;
	if (y < 0)
		begin = -y;
	if (y > 48)
		end = 64 - y;
	shift = 8 - (x & 7);
	if (x < 0)
		bool1 = 0;
	if (x < -8 || x > 119)
		bool2 = 0;
	if (x > 111 || shift == 8)
		bool3 = 0;
	screen = ML_vram_address() + (y + begin << 4) + (x >> 3);
	for (i = begin; i < end; i++)
	{
		line = bmp[i] << shift;
		if (bool1)
			screen[0] |= *p;
		if (bool2)
			screen[1] |= *(p + 1);
		if (bool3)
			screen[2] |= *(p + 2);
		screen += 16;
	}
}
