// Refer to Siomn's fxDocumentation
// https://media.taricorp.net/prizm/simon_chm/fx_legacy_syscalls.htm
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_syscalls.htm
// https://bible.planet-casio.com/simlo/chm/v20/fx_syscall_mother.HTM
// Report any errors in this document to RedCMD

#ifndef _SYSCALL
#define _SYSCALL

#include "dispbios.h"
#include "filebios.h"

typedef enum { SYSCALL_FALSE, SYSCALL_TRUE } SYSCALL_Bool;

// AppName
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_AppName.HTM
char *GetAppName(char *dest);									// SDK, Returns the name of the running application.
int SetAppName(char *src);									// Sets the name of the running application.
int CmpAppName(char *name);									// Compare given name with registered name for the application.

// App
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_app.htm
int App_RegisterAddins();							// scans available G1As and updates the addin array
int FindFreeAddinSlot(void);							// 
//	SYSCALL	000A,	GetAddindHeaderAddr							// 
//	SYSCALL	000E,	App_GetAddindEstripInformation				// Also returns the start of the addin array
int App_LINK_CommandProcessing();					// See fx-9860GII_202.i64 address 800477B0 to have a braf table of the OSes protocol 7.00 processing.
void SetQuitHandler(void (*callback)(void));								// SDK (internal CallbackAtQuitMainFunction)	// Uncommenting causes 'L2300 (E) Duplicate symbol "_SetQuitHandler"' Error
void CallbackAtQuitMainFunction(void (*callback)(void));					// SetQuitHandler
int App_BuiltInCount();																	// returns the number of built in applications
int App_Start(int r4, int r5, int index, int force);									// Jumps to a registered application
void App_CONICS(int runMode, unsigned short stripNum);									// Entrypoint of the CONICS-app
void App_DYNA(int runMode, unsigned short stripNum);									// Entrypoint of the DYNA-app
void App_EACT(int runMode, unsigned short stripNum);									// Entrypoint of the EACT-app
//	SYSCALL	09E1,	App_Equation								// Executes "Equation" (EQUA) built-in program.
void App_EQUA(int runMode, unsigned short stripNum);									// Entrypoint of the EQUA-app, Calls 0x9E1, App_Equation
//	SYSCALL	09F5,	App_Program									// Executes "Program List" (PRGM) built-in program.
void App_FINANCE(int runMode, unsigned short stripNum);									// Entrypoint of the FINANCE-app
int App_GRAPH_TABLE(int r4, int r5);								// Entrypoint of the GRAPH- and TABLE-apps
void App_LINK(int runMode, unsigned short stripNum);									// Entrypoint of the LINK-app
int App_Optimization(void);							// Execute MEMORY's "F5:Optimization" dialog.
void App_MEMORY(int runMode, unsigned short stripNum);									// Execute the "Memory Manager" (MEMORY) built-in program.
void App_RECUR(int runMode, unsigned short stripNum);									// Entrypoint of the RECUR-app
//	SYSCALL	0A97,	App_RUN_MAT_EXE								// Entrypoint, when RUN MAT-app starts analyzing an expression
int App_RUN_MAT(int r4, int r5);									// Entrypoint of the RUN MAT-app
void App_STAT(int runMode, unsigned short stripNum);									// Entrypoint of the STAT-app
void App_SYSTEM(int runMode, unsigned short stripNum);									// Entrypoint of the SYSTEM-app

// Glib
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_GlibGetOSVersionInfo.HTM
//	SYSCALL	0013,	GlibAddinAplExecutionCheck					// SDK
//	SYSCALL	0014,	GlibGetAddinLibInf							// SDK
int GlibGetOSVersionInfo(unsigned char *mainversion, unsigned char *minorversion, unsigned short int *release, unsigned short int *build);						// SDK

// BDisp
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_BDisp.htm
/* typedef struct {
	unsigned int x1;
	unsigned int y1;
	unsigned int x2;
	unsigned int y2;
	unsigned char f[4];
	unsigned int on_bits;
	unsigned int off_bits;
} TShape;
typedef struct {
	int x1;
	int y1;
	int x2;
	int y2;
	unsigned char mode;
} TBdispFillArea; */
//	SYSCALL	001C,	Bdisp_WriteGraph_VRAM						// SDK
//	SYSCALL	001D,	Bdisp_WriteGraph_DD							// SDK
//	SYSCALL	001E,	Bdisp_WriteGraph_DDVRAM						// SDK
//	SYSCALL	0022,	Bdisp_ReadArea_VRAM							// SDK
//	SYSCALL	0024,	Bdisp_GetDisp_DD							// SDK
void Bdisp_PutDisp_DD(void);							// SDK
//	SYSCALL	002A,	Bdisp_DrawShapeToVRAM_002A					// draws a dot, a line, a dashed line, a box or a circle to VRAM
//	SYSCALL	002F,	Bdisp_DrawShapeToVRAM_002F					// draws a dot, a line, a dashed line, a box or a circle to VRAM
//	SYSCALL	0030,	Bdisp_DrawLineVRAM							// SDK
//	SYSCALL	0031,	Bdisp_ClearLineVRAM							// SDK
//	SYSCALL	0032,	Bdisp_DrawShapeToDD							// draws a dot, a line, a dashed line, a box or a circle to DD
//	SYSCALL	0033,	Bdisp_DrawShapeToVRAM_DD					// draws a dot, a line, a dashed line, a box or a circle to VRAM and DD
//	SYSCALL	0034,	Bdisp_DrawShapeToDD_Alternate				// draws a dot, a line, a dashed line, a box or a circle to DD
//	SYSCALL	0035,	Bdisp_DrawShapeToVRAM_DD_Alternate			// draws a dot, a line, a dashed line, a box or a circle to VRAM and DD
//	SYSCALL	011F,	Bdisp_PutDispArea_DD						// (possibly broken)SDK
unsigned char *GetCharacterGlyph(short character);							// see datatables
void Print_OS(unsigned char *msg, int type);									// Prints a string at the current cursor position
//	SYSCALL	0142,	Bdisp_AllClr_DD								// SDK
void Bdisp_AllClr_VRAM( void );							// SDK
//	SYSCALL	0144,	Bdisp_AllClr_DDVRAM							// SDK
//	SYSCALL	0145,	Bdisp_GetDisp_VRAM							// SDK
// void Bdisp_SetPoint_VRAM(int x, int y, unsigned short color);							// SDK
void Bdisp_SetPoint_VRAM(int x, int y, unsigned char point);							// SDK
// void Bdisp_SetPoint_DD(int x, int y, unsigned short color);								// SDK
void Bdisp_SetPoint_DD(int x, int y, unsigned char point);								// SDK
//	SYSCALL	0148,	Bdisp_SetPoint_DDVRAM						// SDK
// unsigned short Bdisp_GetPoint_VRAM( int x, int y );							// SDK
int Bdisp_GetPoint_VRAM(int x, int y);					// SDK
// void Bdisp_AreaClr_DD(TBdispFillArea *p1);				// SDK
void Bdisp_AreaClr_DD(const DISPBOX *pArea);							// SDK
// void Bdisp_AreaClr_VRAM(TBdispFillArea *p1);							// SDK
void Bdisp_AreaClr_VRAM(const DISPBOX *pArea);						// SDK
// void Bdisp_AreaClr_DD( TBdispFillArea*p1 );							// SDK
//	SYSCALL	014C,	Bdisp_AreaClr_DDVRAM						// SDK
// void Bdisp_AreaClr_DDVRAM(const DISPBOX *pArea);		// SDK
//	SYSCALL	014D,	Bdisp_AreaReverseVRAM						// SDK
//	SYSCALL	0150,	PrintXY_0150								// ( int x, int y, unsigned char*msg, int type )
//	SYSCALL	0153,	Disp_Save									// Wrapper for Disp_Manage with act set to 1 (VRAM->RAM loc.)
//	SYSCALL	0154,	Disp_Restore								// Wrapper for Disp_Manage with act set to 0 (RAM loc.->VRAM)
//	SYSCALL	0155,	Disp_GetPtr									// Wrapper for Disp_Manage with act set to 2 (Return ptr to RAM loc)
//	SYSCALL	0156,	PopUpWin_0156								// 
//	SYSCALL	0158,	Disp_Manage									// Function used to copy data to/from VRAM and other display buffers.
void PrintCR(unsigned char *msg, int x, int y);										// ( msg, X, Y )A combination of locate and Print functionality.
void Bdisp_DrawRectangle(int x1, int y1, int x2, int y2);							// draws a rectangle
//	SYSCALL	08FE,	PopupWin_08FE								// ( int nlines )SDK

// RTC
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_RTC.htm
void RTC_Reset(SYSCALL_Bool mode);									// resets RTC
void RTC_GetTime(unsigned int *hour, unsigned int *minute, unsigned int *second, unsigned int *millisecond);									// reads hours, minutes, seconds and milliseconds from RTC
int RTC_GetTicks(void);								// reads RTC-tick-count
int RTC_Elapsed_ms(int start_value, int duration_in_ms);								// checks, if a duration in ms has elapsed
void RTC_SetCarryFlag(void);							// 
void RTC_TriggerAlarm(void);							// Triggers the RTC alarm
void RTC_SetDateTime(unsigned char timestr[8]);								// Set the RTC date and time

// BCD
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_bcd.htm
typedef struct {
	unsigned char hnibble:4;
	unsigned char lnibble:4;
} TBCDbyte;
typedef struct {
	unsigned short exponent:12;
	unsigned short mantissa0:4;
	TBCDbyte mantissa[7];
	char flags;
	short info;
} TBCDvalue;	// 12 (0x0C) bytes
typedef struct {
	int exponent;
	int sign;
	int unknown;
	char mantissa[15];
} TBCDInternal;	// 27 (0x1B) bytes
//	SYSCALL	005C,	Num_UIntToBCD								// Unsigned long up to 0x1869F (99999) to BCD. Func name only a suggestion.
//	SYSCALL	005D,	Num_BCDToUInt								// BCD up to 99999 to unsigned long.
int BCD_GetNotANumberValue(unsigned char *value, unsigned char *result);						// also see NumberFormat
//	SYSCALL	0286,	BCD_AnsToSerial								// (not longer supported since fx-9860GII)Transfers the current answer as string to serial interface with 57600 baud
int BCD_ToStrAsNumber0(char *data, char *string);							// Converts to a number format. Based on syscall 0x4F7.
int BCD_ToStrAsNumber1(char *data, char *string);							// 
int BCD_ToStrAsNumber2(char *data, char *string);							// 
int BCDtoInternal(TBCDInternal *target, TBCDvalue *source);					// 
int BCD_SetAsInt(int input, TBCDvalue *result);								// 
int CalculateExpression(char **expression, char * opcode[2], TBCDvalue *result, int mode);							// See Expressions
void CalculateExpression0(char **expression, char *opcode[2], TBCDvalue *result, int mode, int unknown);			// See Expressions

// Timer
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_timer.htm
int Timer_Install(int InternalTimerID, void *handler, int elapse);								// supports SDK SetTimer
int Timer_Deinstall(int InternalTimerID);								// supports SDK KillTimer
int Timer_Start(int InternalTimerID);									// supports SDK SetTimer
int Timer_Stop(int InternalTimerID);									// supports SDK KillTimer
//	SYSCALL	03F6,	SH7337_TMU_Stop								// 
//	SYSCALL	03F7,	SH7337_TMU_int_handler						// 

// LongToAscHex
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_LongToAscHex.HTM
//	SYSCALL	0467,	LongToAscHex_0467							// Returns an ASCII hexadecimal representation, with given number of digits.
int atoi(const char *value);	// Returns a string's Long representation.
// extern int atoi(const char *);
int LongToAsc(int value, unsigned char *dest, int digits);									// Returns an ASCII representation, with given ASCII string.
int LongToAscHex(int value, char *dest, int digits);								// The same functionality like 0x467

// Hardware_Aspects
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_Hardware.HTM
void DiagnosticMode();								// 
//	SYSCALL	0CD0,	diagnostic_dialog							// 
//	SYSCALL	0468,	Check_for_SD								// -optionreturns 1, if SD; returns 0, if non-SD (uses bit 0 of port E)
void TestMode();									// Enters the "TEST MODE" program, also avail via keypresses.

// SMEM
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_SMEM.HTM
//	SYSCALL	019F,	StorageMemory_Optimization					// 
//	SYSCALL	020E,	StorageMemory_GetFilePos					// retrieves the current position of an opened storage memory file

// Keyboard
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_keyboard.htm
void Keyboard_ClrBuffer(void);							// clears the keyboardbuffer
// void Bkey_Set_RepeatTime( int FirstCount, int NextCount );							// SDK
void Bkey_Set_RepeatTime(long FirstCount, long NextCount);							// SDK
// void Bkey_Get_RepeatTime( int*FirstCount, int*NextCount );							// SDK
void Bkey_Get_RepeatTime(long *FirstCount, long *NextCount);	// SDK
void Bkey_Set_RepeatTime_Default( void );					// SDK
void Keyboard_EnableAutoRepeat( void );					// 
void Keyboard_DisableAutoRepeat( void );					// 
int Keyboard_GetKeyWait(int *column, int *row, int type_of_waiting, int timeout_period, SYSCALL_Bool menu, unsigned short *keycode);							// SDK, broken?
void GetKeyWait_OS(int*column, int*row, int type_of_waiting, int timeout_period, int menu, unsigned short*keycode );								// 
int Keyboard_PutKeyCode(int X, int Y, int KeyCode);							// Injects a keycode into the key-buffer
//	SYSCALL	0249,	Keyboard_GetKeyDownTime						// 
//	SYSCALL	024A,	Keyboard_IsAnyKeyDown						// returns the matrix code of the actual pressed key
//	SYSCALL	024B,	Keyboard_IsSpecialKeyDown_024B				// checks if the matrix code of a special key is pressed (matrix code variant of IsKeyDown)
//	SYSCALL	024C,	Keyboard_IsSpecialKeyDown_024C				// SDK, 0x024B with doubled poll-count, accounts for chattering
int Keyboard_KeyDown(void);							// directly queries port A
void Keyboard_SecondaryInterruptHandler(void);			// 
//	SYSCALL	024F,	Keyboard_PutKeymatrixCode
int Keyboard_PutKeymatrixCode(short *matrixCode);			// Injects a keymatrixcode into the key-buffer
void Keyboard_TimerHandler(void);						// 
void Keyboard_PrimaryInterruptHandler(void);			// 
//	SYSCALL	03EA,	Datatable_F_key_display_bitmaps				// Starting with offset 0x0822 (OS 1.03) inside this table the F-key-display-bitmaps are stored. Those bitmaps consist of 24 bytes. The first 21 bits of three adjacent bytes define a horizontal line.
void EnableGetkeyToMainFunctionReturn(void);			// 
void DisableGetkeyToMainFunctionReturn(void);			// 
//	SYSCALL	04CB,	Keyboard_RemapFKeyCode_04CB					// changes the GetKey-returncode of a set of FKeys
int Keyboard_PRGM_GetKey(char result[12]);						// PRGM-GetKey (non-blocking and calc-type independent)
int GetKey(unsigned int *keycode);										// SDK
void PutKey(unsigned int keycode, SYSCALL_Bool mode);										// Puts a GetKey-keycode into keybuffer
//	SYSCALL	0A1F,	Keyboard_RemapFKeyCode_0A1F					// changes the GetKey-returncode of a set of FKeys; uses 0x4CB
int Bkey_GetKeyWait(int *kcode1, int *kcode2, int type_of_waiting, int timeout_period, SYSCALL_Bool menu, unsigned short *unused); // SDK

// Auxiliary_functions
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_AUX.htm
void GetFKeyIconPointer(int FKeyNo, unsigned char *pBitmap);							// 
int GetIntPtrContent(unsigned char *ptr);							// convert non-4-aligned 4-byte sequences to int
//	SYSCALL	04A0,	AUX_DisplayErrorMessage						// like 0x954. Reboots if parameter is 23.
//	SYSCALL	04A0,	DisplayErrorMessage						// like 0x954. Reboots if parameter is 23.
//	SYSCALL	04B0,	AUX_DisplayFKeyIcons						// 
//	SYSCALL	04B0,	DisplayFKeyIcons						// 
//	SYSCALL	04D1,	AUX_DisplayFKeyIcon							// 
//	SYSCALL	04D1,	DisplayFKeyIcon							// 
void HourGlass();									// 
void EditMBStringCtrl( unsigned char*, int xposmax, void*, void*xpos, void*key, int x, int y );							// 
void DisplayMBString( unsigned char*, int, int xpos, int x, int y );								// 
void EditMBStringChar(unsigned char *, int xposmax, int xpos, int character);	// 
//	SYSCALL	0901,	AUX_DisplayMessageBox						// 
//	SYSCALL	0905,	AUX_DisplayErrorMessage_0905				// Displays an error message box depending on the int parameter passed.
//	SYSCALL	0954,	AUX_DisplayErrorMessage_0954				// Displays an error message box depending on the int parameter passed.
//	SYSCALL	0A35,	AUX_DisplayMessage							// Displays a four line message box depending on the int parameters passed.
//	SYSCALL	0CC4,	Number_Input_Dialog							// 
//	SYSCALL	0CC5,	String_Input_Dialog							// 
int InputDateDialog(int *day, int *month, int *year, int *P4);								// ( int*, int*, int*, int* )
int InputMonthDialog(int *month, int *P2);							// ( int*, int* )
int InputDayDialog(int *day, int *P2);								// ( int*, int* )
int InputYearDialog(int *year, int *P2);								// ( int*, int* )

// Serial_Communication
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_Serial.htm
int Serial_Open2(unsigned short parameters);								// opens the serial channel, unsigned short parameter
int Serial_Open_57600(void);							// (not longer supported since fx-9860GII)opens the serial channel with 57600 baud
void *Serial_ResetAndDisable(void);						// 
void *Serial_GetInterruptHandler(int type);					// 
int Serial_SetInterruptHandler(int type, void *handler);					// 
int Serial_ReadOneByte(unsigned char *result);							// fetches the next available byte from the serial receive interrupt buffer
int Serial_ReadNBytes(unsigned char *result, int max_size, short *actually_transferred);							// fetches a specified count of bytes from the serial receive interrupt buffer
int Serial_BufferedTransmitOneByte(unsigned char byte_to_transmit);				// drops a byte into the serial transmit interrupt buffer
int Serial_BufferedTransmitNBytes(unsigned char *bytes_to_transmit, int requested_count);				// drops N bytes into the serial transmit interrupt buffer
int Serial_DirectTransmitOneByte(unsigned char byte_to_transmit);				// drops a byte into the serial transmit FIFO
int Serial_GetReceivedBytesAvailable(void);			// returns the count of bytes available in the receive buffer
int Serial_GetFreeTransmitSpace(void);					// returns free transmit space
int Serial_ClearReceiveBuffer(void);					// clears the serial receive interrupt buffer
int Serial_ClearTransmitBuffer(void);					// clears the serial transmit interrupt buffer
int Serial_Open(unsigned char *mode);									// opens the serial channel
int Serial_Close(SYSCALL_Bool mode);								// closes the serial channel
void *Serial_CallReceiveIntErrorResetHandler(void);		// 
void *Serial_CallReceiveIntHandler(void);				// 
void *Serial_CallTransmitIntErrorResetHandler(void);		// 
void *Serial_CallTransmitIntHandler(void);				// 
int Serial_SpyNthByte(int byteno_to_spy, unsigned char *result);							// Spies the Nth byte from the serial receive interrupt buffer, if available
void Serial_GetStatus(unsigned int *serial_status);							// OS; get serial status from SCSSR (function changes the original bit order)
int Serial_IsOpen(void);								// queries the open-status of the serial channel

// Communication
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_Communication.htm
typedef struct{
	unsigned char type;
	unsigned char subtype;
	unsigned short datasize;
	void*databuffer;
	char checksum;
} TReceivePacketDesc;
int Comm_Open(unsigned short parameters);									// opens a communication's channel
int Comm_Close(int mode);									// closes the open communication's channel
int Comm_WaitForAnyBuffer(int timeout_ms, int P2, int *P3);						// 
int Comm_ReadOneByte(unsigned char *result);							// fetches the next available byte from the open communication's receive buffer
//	SYSCALL	0291,	Comm_TransmitOneByte						// drops a byte into the open communication's transmit buffer
int Comm_WaitForAndReadNBytes(char *buffer, int bytes_to_read);	// waits for and fetches N available bytes from the open communication's receive buffer
//	SYSCALL	0293,	Comm_TransmitNBytes							// drops N bytes into the open communication's transmit buffer
//	SYSCALL	0294,	Comm_ClearReceiveBuffer						// clears the open communication's receive interrupt buffers
//	SYSCALL	0295,	Comm_ClearTransmitBuffer					// clears the open communication's transmit interrupt buffers
int Comm_IsValidPacketAvailable(unsigned char *result);					// spies for a byte in the receivebuffer, which is a valid packet type
//	SYSCALL	0298,	Comm_IsOpen									// queries the open-status of the communication's channel
int Comm_GetCurrentSelector(void);						// returns the current selected communication's channel (USB or serial)
int Comm_Padding_5C(unsigned char *source, unsigned short source_count, unsigned char *target, unsigned short *target_count);								// 
int Comm_ReversePadding_5C(unsigned char *source, unsigned char *target, unsigned short source_count, unsigned short *target_count);	// 
int Comm_Spy0thByte(unsigned char *result);								// 
int Comm_ProcessInPacket(TReceivePacketDesc *rpd);						// Retrieve the next packet
int Comm_PrepareAckPacket(TReceivePacketDesc *rpd, unsigned char subtype, void *datapointer, unsigned short datasize);						// 
void Comm_PrepareErrorPacket(TReceivePacketDesc *rpd, unsigned char subtype);						// 
void Comm_PrepareTerminatePacket(TReceivePacketDesc *rpd, unsigned char subtype);					// 
void Comm_PrepareRoleswapPacket(TReceivePacketDesc *rpd, unsigned char subtype);					// 
void Comm_PrepareCheckPacket(TReceivePacketDesc *rpd, unsigned char subtype);						// 
int Comm_PrepareCommandPacket(TReceivePacketDesc *rpd, unsigned char subtype, void *datapointer, unsigned short datasize);					// 
int Comm_PrepareDataPacket(TReceivePacketDesc *rpd, unsigned char subtype, void *datapointer, unsigned short datasize);						// 
void USB_TimerHandler(void);							// 

// Converting
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_Converting.HTM
int HexToByte(short *Hex_Value, char *result);									// returns 1, if successful. returns -1, if not successful
int HexToWord(int *Hex_Value, short *result);									// returns 1, if successful. returns -1, if not successful
//	SYSCALL	02A3,	ByteToHex									// Returns hex representation (ascii) of given byte into a char[2].
//	SYSCALL	02A4,	WordToHex									// ( unsigned short w, unsigned short*buffer )Returns hex rep. (ascii) of given word into a char[4].
//	SYSCALL	02A7,	AscHexToNibble								// Sets result to 0xFF, if Hex Value is invalid.
void HexToNibble(char Hex_Value, char *result);									// Sets result to 0xFF, if Hex Value is invalid.
//	SYSCALL	02A8,	NibbleToAscHex								// Returns hex rep. (ascii) of given nibble into a char. Only used by ByteToAscHex
//	SYSCALL	02A8,	NibbleToHex									// Returns hex rep. (ascii) of given nibble into a char. Only used by ByteToAscHex
//	SYSCALL	0541,	itoa										// Convert provided int (r4) to the provided zero-terminated string (r5). Base 10.

// System
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_System.HTM
int System_UpdateOS(void);								// OS update
// void System_GetOSVersion(unsigned char version[10]);							// Returns the OS version as ASCIIZ string in version.
void System_GetOSVersion(char version[10]);							// Returns the OS version as ASCIIZ string in version.

// MCS
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_MCS.htm
typedef struct {
	unsigned char name[8];
	void*addr;
	unsigned short count;
	char flags[2];
} TMainMemoryDirectoryEntry;
// TMainMemoryDirectoryEntry MainMemoryDirectory[147];	// 0x93
typedef struct {
	unsigned char name[8];
	int offset;
	int length;
	char flags[4];
} TDirectoryItem;
// TDirectoryItem ItemArray[MainMemoryDirectory[i].count];
int MCS_CreateDirectory(unsigned char *dir, char *dirno);							// 
int MCS_WriteItem(unsigned char *dir, unsigned char *item, short itemtype, int data_length, int buffer);								// 
int MCS_DeleteDirectory(unsigned char *dir);							// 
int MCS_DeleteItem(unsigned char *dir, unsigned char *item);								// 
int MCS_GetState(int *maxspace, int *currentload, int *remainingspace);								// 
int MCS_GetSystemDirectoryInfo(unsigned char *dir, unsigned int *pdir, char *dirno);					// 
int MCS_RenameItem(unsigned char *srcdir, unsigned char *srcitem, unsigned char *tgtdir, int tgtitem);								// (MoveItem)
int MCS_OverwriteData(unsigned char *dir, unsigned char *item, int write_offset, int bytes_to_write, void *buffer);							// 
int MCS_GetItemData(unsigned char *dir, unsigned char *item, int offset, int bytes_to_read, void *buffer);								// 
int MCS_RenameDirectory(unsigned char *oldname, unsigned char *newname);							// 
//	SYSCALL	0374,	BMCSRenameVariable							// SDK
int MCS_SearchDirectory(unsigned char *dir, TMainMemoryDirectoryEntry *pdir, char *dirno);							// 
int MCS_SearchDirectoryItem(unsigned char *dir, unsigned char *item, char *flags_0, TDirectoryItem *item_ptr, int *data_ptr, int *data_length);	// 
int MCS_GetFirstDataPointerByDirno(char *dirno, void *pdata);				// 
int MCS_GetDirectoryEntryByNumber(char dirno, TMainMemoryDirectoryEntry *pdir);				// 
int MCS_SearchItem(unsigned char *item, TMainMemoryDirectoryEntry *pdir, unsigned short *itemno);								// 
int MCS_str8cpy(unsigned char *source, unsigned char *target, SYSCALL_Bool mode);									// 
void MCS_GetDirectoryEntryAddress(void *directory_entry_address);				// 
void MCS_GetCurrentBottomAddress(void *current_bottom_address);					// 
int MCS_GetCapa(int *current_bottom);									// 
int MCS_GetMainMemoryStart(void);						// 

// MCS_Handles
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_MCShandles.HTM
int CLIP_Store(unsigned char *buffer, int length);									// stores a buffer to the clipboard
//	SYSCALL	0829,	MCS_CreateDirectory_0829					// 
int MCS_PutInternalItem(char dirtype, unsigned char *item, int data_len, void *buffer);							// 
int MCSPutVar2(unsigned char *dir, unsigned char *item, int data_len, void *buffer);									// SDK
int MCSOvwDat2(unsigned char *dir, unsigned char *item, int bytes_to_write, void *buffer, int write_offset);									// SDK
int MCS_OverwriteOpenItem(int bytes_to_write, void *buffer, int write_offset);						// 
int MCS_ClearInternalDirectory(char dirtype);					// 
int MCS_ClearDirectory(unsigned char *dir);							// 
int MCS_DeleteInternalItem(char dirtype, unsigned char *item);						// 
int MCSDelVar2(unsigned char *dir, unsigned char *item);									// SDK
int MCS_GotoInternalItem(char dirtype, unsigned char *item, int direction);						// 
int MCS_OpenMainMemoryItem(unsigned char *dir, unsigned char *item, char direction);						// 
int MCS_GotoHandleNeighbour(char direction);						// 
int MCS_CheckOpenedItem(char *dirtype, int *data_len);							// 
int MCS_GetOpenItem(unsigned char *item);								// 
int MCS_OpenInternalDirectoryItem(char dirtype, unsigned char *item, int *data_len);				// 
int MCSGetDlen2(unsigned char *dir, unsigned char *item, int *data_len);									// SDK
int MCSGetData1(int offset, int len_to_copy, void *buffer);									// SDK
int MCS_MapMCS_Result(void);							// 
//	SYSCALL	0844,	MCSGetCapa									// SDK
int MCS_OpenInternalDirectoryItem2(char dirtype, unsigned char *item, int *data_len, int *data_ptr);				// 
int MCS_OpenAlphaMemItem(char variablename, int *data_len, void *data_ptr);						// 
int MCS_DirtypeToItemtype(char dirtype);						// 
int MCS_ItemtypeToDirtype(char itemtype);						// 
unsigned char *MCS_DirtypeToName(char dirtype);							// 
int MCS_MapError(int error_in);								// 
int MCS_DeleteDirectoryItems(unsigned char *dir);					// 

// Setup
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_Setup.htm
typedef struct {
	unsigned char Name1[13];	// will be set to "SETUP.g1m" on return
	unsigned char Name2[9];		// will be set to "$GLOBAL" on return
	unsigned char Name3[10];	// will be set to "SETUP" on return
	void *StartOfSetup;			// 
	unsigned int LengthOfSetup;	// 
	unsigned short unknown;		// 0x8038
} TSetupInfo;
int Setup_GetInfo(unsigned char *P1, int P2, TSetupInfo *P3);	// ( const unsigned char*, int, TSetupInfo* )
char Setup_GetEntry(unsigned int index);						// Returns the Setup value at a given index.
char *Setup_SetEntry(unsigned int index, char value);			// Sets the Setup value at a given index.
char *Setup_GetEntryPtr(unsigned int index);					// Returns a pointer to Setup data with specific index.
//	SYSCALL	0518,	Setup_GetEntry_3E							// as short variable
//	SYSCALL	0519,	Setup_GetEntry_40							// as short variable
//	SYSCALL	051A,	Setup_SetEntry_3E							// as short variable
//	SYSCALL	051B,	Setup_SetEntry_40							// as short variable
//	SYSCALL	090B,	SetShiftAlphaState							// See Setup
//	SYSCALL	090C,	GetInsOverwriteState						// See Setup
//	SYSCALL	090D,	SetInsOverwriteState						// See Setup
//	SYSCALL	090E,	ClrShiftAlphaState							// See Setup
//	SYSCALL	091B,	GetShiftAlphaState							// See Setup

// Interrupt
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_INTERRUPT.HTM
int Interrupt_SetOrClrStatusFlags(int flagmask, int set);				// 
//	SYSCALL	04AD,	USB_InterruptHandler						// 

// Sleep
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_Sleep.HTM
//	SYSCALL	0037,	Wait_ms_TMU_2								// ( duration )wait duration ms (uses TMU_2; max. duration=2000)
//	SYSCALL	0130,	Wait_ms_WDT									// ( duration )wait duration ms (uses WDT)
void Sleep(int delay_ms);										// 
//	SYSCALL	0420,	OS_inner_Sleep								// ( int delay_ms );
void OS_InnerWait_ms(int delay_ms);								// ( int delay_ms );

// BFile
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_Bfile.htm
int Bfile_identify_device_OS(const FONTCHARACTER *filename);					// Tries to detect the device referenced by a FONTCHARACTER-filename.
int Bfile_OpenFile_OS(const FONTCHARACTER *filename, int mode, int mode2);							// OS (SDK)
int Bfile_CloseFile_OS(int HANDLE);							// OS (SDK)
int Bfile_GetMediaFree_OS(unsigned short *media_id, int *freespace);						// OS (SDK); incompatible since series GII-2
int Bfile_GetFileSize_OS(int HANDLE);						// OS (SDK)
int Bfile_SeekFile_OS( int HANDLE, int pos );							// OS (SDK)
int Bfile_ReadFile_OS(int HANDLE, void *buf, int size, int readpos);							// OS (SDK)
int Bfile_CreateEntry_OS(const FONTCHARACTER *filename, int mode, int *size);						// OS (SDK)
int Bfile_WriteFile_OS(int HANDLE, const void *buf, int size);							// OS (SDK)
int Bfile_RenameEntry(const unsigned short *oldname, const unsigned short *newname);							// OS (SDK)
int Bfile_DeleteEntry(const FONTCHARACTER *filename, int mode);							// OS (SDK)
int Bfile_FindFirst( const FONTCHARACTER *pathname, int *FindHandle, FONTCHARACTER *foundfile, FILE_INFO *fileinfo );								// SDK
int Bfile_FindNext( int FindHandle, FONTCHARACTER *foundfile, FILE_INFO *fileinfo );								// SDK
int Bfile_FindClose( int FindHandle );								// SDK
//	SYSCALL	0450,	Bfile_GetFilenameLength						// 
//	SYSCALL	0451,	Bfile_Name_cmp								// Compares two FONTCHARACTER( short )-arrays
//	SYSCALL	0452,	Bfile_Name_cpy								// Copies two FONTCHARACTER( short )-arrays
//	SYSCALL	0453,	Bfile_Name_ncpy								// Copies n characters of two FONTCHARACTER( short )-arrays
//	SYSCALL	0456,	Bfile_NameToStr_ncpy						// Copies a FONTCHARACTER( short )-array to a char array
void Bfile_StrToName_ncpy(unsigned short *dest, const unsigned char *source, int n);						// Copies a char array to a FONTCHARACTER( short )-array

// Battery
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_battery.htm
//	SYSCALL	0469,	Battery_DisplayLowStatus_0469				// 
//	SYSCALL	0476,	Battery_IsLow_0476							// 
//	SYSCALL	0492,	Battery_IsLow_0492							// 
//	SYSCALL	0495,	Battery_DisplayLowStatus_0495				// 
int Battery_GetStatus(int delayflag);							// 

// Heap
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_heap.htm
int Heap_SetTopChunk(int size);							// 

// MultiByte
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_mb.htm
int MB_IsLead(char character);									// does byte signal an extended character set?
int MB_ElementCount(char *str);								// number of elements in multibyte string (number of symbols that will be "printed")
int MB_ByteCount(char *str);								// number of bytes in multibyte string (number of bytes that must be copied)
char *MB_strcat(char *dest, char *src);									// concatenate two multibyte strings
char *MB_strncat(char *dest, char *src, int bytes);									// same, but with size limitation
char *MB_strcpy(char *dest, char *src);									// copy a multibyte string
char *MB_GetSecondElemPtr(char *str);							// get pointer to second element in string
short int MB_GetElement(char *str);								// returns value (a short), containing first element in string
char *MB_CopyToHeap(char *src);								// allocates memory and copies provided string to the heap. NOT used by OS

// PRGM
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_PRGM.HTM
//	SYSCALL	0652,	NextOpcode									// PRGM related
//	SYSCALL	0652,	PRGM_NextOPcode								// 
//	SYSCALL	06A6,	IsEndOfLine									// PRGM related
//	SYSCALL	0CA7,	char_OpcodeType								// ( char* );
char OpcodeType( char* );									// 
//	SYSCALL	0CB0,	Send										// (/Send38kPRGM related
//	SYSCALL	0CB1,	Receive										// (/Receive38kPRGM related
//	SYSCALL	0CB2,	Open										// /CloseComPort38kPRGM related

// Alpha_Memory
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_AlphaMem.htm
char *Alpha_GetData(char variable, char *dest);								// Returns the data of a given AlphaMem variable.
char Alpha_SetData(char variable, char *src);								// Copies the given data to an AlphaMem variable.
void Alpha_ClearAll(void);								// Clears all AlphaMem variables, except Ans.
int Alpha_GetData2(char variablecode, char *datadest);								// Returns the data of a given AlphaMem variable and does a range check.
void Alpha_ClearAllAndAns();						// Clears all AlphaMem variables.
//	SYSCALL	090B,	SetShiftAlphaState							// See Setup
//	SYSCALL	090E,	ClrShiftAlphaState							// See Setup
//	SYSCALL	091B,	GetShiftAlphaState							// See Setup

// Expressions
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_Expressions.HTM
int CalculateExpression(char **expression, char *opcode[2], TBCDvalue *result, int mode);	// See BCD
//	SYSCALL	064A,	CalculateExpression0						// 
//	SYSCALL	07FC,	Expressions_OpcodeToStr						// 
void OpCodeStrPtr( int hcode, int lcode );								// 
void DisplayMBString2(int P1, unsigned char *buffer, int P3, int xpos, int zero, int x, int y, int pos_to_clear, int P9, int P10);							// 
int EditValue(int mode, short key, int row, TBCDvalue *value, char *pretext);									// 
int EditExpression(int mode, short key, int row, TBCDvalue *value, char *editbuffer, short editbufferlen, char *pretext, int mode2);								// 
//	SYSCALL	0AE8,	Catalog										// dialogreturns opcode
//	SYSCALL	0AE8,	Catalog_dialog								// 
void StoreExpressionToGraphFuncMemory(int funo, int unknown, unsigned char *expression);			// 

// Number_Format
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_numberformat.htm
//	SYSCALL	04F0,	BCD_ToStrAsNumber0							// Converts to a number format. Based on syscall 0x4F7.
//	SYSCALL	04F5,	BCD_ToStrAsNumber1							// BCD
//	SYSCALL	04F6,	BCD_ToStrAsNumber2							// 
void Num_DataToStr(char *data, char *string, int *unkown);								// (char *data, char *string, int *unkown)
void Num_DataToStrAsFrac(char *data, char *string);							// Converts a BCD-value into a string using the frac-format. Setup-entry 0x57 controls the format.
void Num_DataToStr_VARIANT5(char *data, char *string);						// Converts to an angle format. Based on syscall 0x4F7.
//	SYSCALL	0500,	BCDToInternal								// 

// Math
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_Math.HTM
unsigned char *Math_GetConstant(unsigned char *buffer);							// 12byte buffer. Returns a standard BCD-coded value in the first 9 bytes. The last 3 bytes are insignificant.
unsigned char *Math_GetConstant_0(unsigned char *buffer);						// 0
unsigned char *Math_GetConstant_1(unsigned char *buffer);						// 1
unsigned char *Math_GetConstant_2(unsigned char *buffer);						// 2
unsigned char *Math_GetConstant_10(unsigned char *buffer);						// 10
unsigned char *Math_GetConstant_1_3(unsigned char *buffer);						// 1/3 (not BCD coded, special FX format)
unsigned char *Math_GetConstant_0_5(unsigned char *buffer);						// 0.5
unsigned char *Math_GetConstant_32767(unsigned char *buffer);					// 32767
unsigned char *Math_GetConstant__32768(unsigned char *buffer);					// -32768
unsigned char *Math_GetConstant_65536(unsigned char *buffer);					// 65536
unsigned char *Math_GetConstant_0x7FFFFFFF(unsigned char *buffer);				// 0x7FFFFFFF
unsigned char *Math_GetConstant__2Giga(unsigned char *buffer);					// -2Giga
unsigned char *Math_GetConstant_4Giga(unsigned char *buffer);					// 4Giga
unsigned char *Math_GetConstant_pi(unsigned char *buffer);						// pi
unsigned char *Math_GetConstant_2pi(unsigned char *buffer);						// 2*pi
unsigned char *Math_GetConstant_pi_2(unsigned char *buffer);					// pi/2
unsigned char *Math_GetConstant_e(unsigned char *buffer);						// e
unsigned char *Math_GetConstant_5(unsigned char *buffer);						// 5
unsigned char *Math_GetConstant_pi_4(unsigned char *buffer);					// pi/4
unsigned char *Math_GetConstant_ln_10(unsigned char *buffer);					// ln 10
unsigned char *Math_GetConstant_ln_2(unsigned char *buffer);					// ln 2
unsigned char *Math_GetConstant_9_99E99(unsigned char *buffer);					// 9.99E99
unsigned char *Math_GetConstant__9_99E99(unsigned char *buffer);				// -9.99E99
unsigned char *Math_GetConstant_9_99999999999999E99(unsigned char *buffer);		// 9.99999999999999E99
unsigned char *Math_GetConstant_227_85(unsigned char *buffer);					// 227.85
unsigned char *Math_GetConstant_SQRT2(unsigned char *buffer);					// SQRT(2)
unsigned char *Math_GetConstant_SQRT2_2(unsigned char *buffer);					// SQRT(2)/2
unsigned char *Math_GetConstant_506_6282746310(unsigned char *buffer);			// 506.6282746310

// Cursor
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_Cursor.HTM
int Cursor_SetPosition(char column, char row);							// Sets the cursor to provided column and row.
int Cursor_SetFlashStyle(short int flashstyle);						// Sets the look/style of the flashing cursor.
void Cursor_SetFlashMode(long flashmode);							// Configures the visibility of a flashing cursor
unsigned int Cursor_GetSettings(unsigned int settingsarray[]);							// Used to retrieve the current Cursor settings.
void locate(int column, int row);										// SDK, OS
int Cursor_GetFlashStyle(void);						// returns the current flash style fitting for a call of 0x811.
//	SYSCALL	080F,	Cursor_GetSettings_Alternate				// ( TCursorSettings*CursorSettings );
void Cursor_SetFlashOn(char flash_style);							// Enables cursor-flashing and sets the look/style of the cursor.
void Cursor_SetFlashOff(void);							// Turns off the cursor flashing.

// Datatable
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_datatable.HTM
char *GetCharacterMiniGlyph(short glyphno, short *width);						// see datatables
//	SYSCALL	0175,	Datatable_English_Messages					// defines the English messages. Offset 0x0C defines count-1 of messages (short). At offset 0x0D an according list of short offsets is positioned. These short-offsets point to the actual messages, relative from after the end of this short-offset-list.
//	SYSCALL	01B7,	Get8x8BitmapPointer							// Return a bitmap out of the 8x8-font-datatables
unsigned char *Get8x8BitmapPtr1(int bitmapno);							// returns a selected pointer to a 8x8 bitmap out of the font-/symbol-table 1
unsigned char *Get8x8BitmapPtr01(void);							// returns a pointer to a 8x8 bitmap which is actually empty (clear)
unsigned char *Get8x8BitmapPtr02(void);							// returns a pointer to a 8x8 bitmap which is actually empty (clear)
unsigned char *Get8x8BitmapPtr2(int bitmapno);							// returns a selected pointer to a 8x8 bitmap out of the font-/symbol-table 2
unsigned char *Get8x8BitmapPtr3(int bitmapno);							// returns a selected pointer to a 8x8 bitmap out of the font-/symbol-table 3
unsigned char *Get8x8BitmapPtr4(int bitmapno);							// returns a selected pointer to a 8x8 bitmap out of the font-/symbol-table 4
unsigned char *Get8x8BitmapPtr03(void);							// returns a pointer to a 8x8 bitmap which is actually empty (clear)
//	SYSCALL	0486,	GetdatatablePtr								// returns pointers to OS message- and bitmap-datatables

// DD
//	SYSCALL	001B,	DD_Clear									// 
//	SYSCALL	0025,	DD_Read										// Reads the data line of the DD (often the next page in the DD RAM, depending on conf).
//	SYSCALL	0026,	DD_ReadFromPage								// Select page and return the data. SDK - renamed from DD_GET
//	SYSCALL	0027,	DD_WriteToPage								// Select page and write supplied data.
void DD_Poweroff(void);											// Turns off the DD.
int DD_SetContrast(int value);									//
//	SYSCALL	0133,	DD_SetFRS									// Set the DD FRS.
//	SYSCALL	0134,	DD_SetBias									// Set the DD bias.

// Power
//	SYSCALL	0236,	RebootOS_0236								// 'Reboots' the hardware/OS
void PowerOff(int logo);										// r4=0: Poweroff. r4=1: Display logo and poweroff.
//	SYSCALL	047F,	SetAutoPowerOffTime							// in minutes
//	SYSCALL	049E,	RebootOS_049E								// 'Reboots' the hardware/OS Calls 0x236, RebootOS
int GetAutoPowerOffTime(void);									 // in minutes
//	SYSCALL	048D,	SetAutoPowerOffFlag							// 1=AutoPowerOff
//	SYSCALL	048E,	GetAutoPowerOffFlag							// 1=AutoPowerOff

// Print
//	SYSCALL	0713,	Print_ClearLine_OS							// Clears a line
//	SYSCALL	0808,	Print										// SDK
//	SYSCALL	0809,	PrintRev									// SDK
//	SYSCALL	080A,	PrintC										// SDK
//	SYSCALL	080B,	PrintRevC									// SDK
// void PrintLine(unsigned char *msg, int imax); 					// SDK
void PrintLine(const unsigned char *str, int max);				// SDK
//	SYSCALL	080D,	PrintRLine									// SDK
//	SYSCALL	09AD,	PrintXY_09AD								// SDK
//	SYSCALL	0C4F,	PrintMiniSd									// SDK

// Memory
// https://bible.planet-casio.com/simlo/chm/v20/fx_legacy_fastvramcopy.HTM
#ifndef _SIZE_T
#define _SIZE_T
typedef unsigned long size_t;
#endif
unsigned int VRam_Base(void);									// Returns the address of the VRAM.
char *GetVRAMAddress(void);										// Returns the address of the VRAM.
//	SYSCALL	0404,	GetPhysicalROMstart							// 
//	SYSCALL	0405,	GetPhysicalRAMstart							// 
int GetRAMSize( void );											// ( void )returns 0x40000 or 0x80000
//	SYSCALL	03F5,	ClearMainMemory								// Clears main memory, and reboots, showing 'Main memories cleared'.
//	SYSCALL	0172,	strcmp_case_insensitive						// case-insensitive
//	SYSCALL	0173,	strcmp_0172									// branches to syscall 0x172
void InvertMem( void*memoryrange, int count );									// inverts a memory range
size_t strlen(const char *s);										// string
//	SYSCALL	02AA,	memcpy_slow									// string, slow version - should probably use 0xACF
//	SYSCALL	035E,	memset_range								// 
//	SYSCALL	035F,	another_memset								//
void *memcpy(void *s1, const void *s2, size_t n); //
//	SYSCALL	044F,	another_memcmp_044F							// int memcmp(const void *s1, const void *s2, size_t n)
//	SYSCALL	053F,	another_memcmp_053F							// 
void free(void *);										// SDK, stdlib
void *malloc(size_t);										// SDK, stdlib
int memcmp(const void *s1, const void *s2, size_t n);										// string
//	SYSCALL	0ACF,	memcpy_smart								// smart memcpy; uses mov.b, mov.w or mov.l depending on parameter alignment
void *memset(void *s, int c, size_t n);										// string
char *strcat(char *s1, const char *s2);										// string
//	SYSCALL	0AD5,	strcmp_smart								// smart strcmp; uses mov.b or mov.l depending on parameter alignment
//	SYSCALL	0AD6,	strlen_Alternate							// string
char *strncat(char *s1, const char *s2, size_t n);										// string
int strncmp(const char *s1, const char *s2, size_t n);										// string
//	SYSCALL	0AD9,	strncpy										// string
//	SYSCALL	0ADA,	strrchr										// string
void *calloc(size_t, size_t);										// SDK, stdlib
void *memmove(void *s1, const void *s2, size_t n);										// 
void *realloc(void *, size_t);										// SDK, stdlib
//	SYSCALL	0E6E,	strchr										// 
//	SYSCALL	0E6F,	strstr										// 

// System_Calls
void SysCall_Name(void);									// Description
int Calljmp(int r4, int r5, int r6, int r7, int address);	// 
int SysCalljmp(int r4, int r5, int r6, int r7, int number);	// 
int SysCalljmp12(int r1, int r2, int r3, int r4, int r5, int r6, int r7, int r8, int r9, int r10, int r11, int r12, int number);	// 

// Other
//	SYSCALL	0001,	TLB_exception_handler_protection			// (protection)
//	SYSCALL	0002,	CPU_address_error_handler					// (instruction/data)
//	SYSCALL	0003,	TLB_exception_handler_page_write			// (miss/invalid/initial page write)
//	SYSCALL	0018,	MMU_FlushCache_0018							// uses the 0xA0000000-based uncached address
void pc_toupper( char* );									// ( char* )
void pc_tolower( char* );									// ( char* )
void GUI_ProgressBar(int current, int max);								// Displays a progressbar dialog ('One moment please'), and can be used to draw the bar.
//	SYSCALL	03FA,	Hmem_SetMMU									// SDK
//	SYSCALL	03FE,	GetStackPtr									// 
//	SYSCALL	03FF,	MMU_FlushCache_03FF							// uses the 0x80000000-based cached address
//	SYSCALL	04E9,	LocalizeStringID							// Returns pointer to localized string of given ID.
//	SYSCALL	0542,	to_uppercase								// Convert 'a'-'z' byte at provided pointer destination, to uppercase.
//	SYSCALL	0543,	to_lowercase								// Convert 'A'-'Z' byte at provided pointer destination, to lowercase
//	SYSCALL	0813,	SaveDisp									// SDK
//	SYSCALL	0814,	RestoreDisp									// SDK

#endif	// _SYSCALL